//------------------------------------------------------------------------
//  COLORS
//------------------------------------------------------------------------
//
//  Eureka DOOM Editor
//
//  Copyright (C) 2001-2009 Andrew Apted
//  Copyright (C) 1997-2003 Andr Majorel et al
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//------------------------------------------------------------------------
//
//  Based on Yadex which incorporated code from DEU 5.21 that was put
//  in the public domain in 1994 by Raphal Quinet and Brendon Wyber.
//
//------------------------------------------------------------------------

#include "main.h"

#include "im_color.h"
#include "im_img.h"  // TRANS_PIXEL
#include "r_misc.h"
#include "w_wad.h"


pcolour_t palette[256];

int trans_replace;

byte raw_palette[256][3];
byte raw_colormap[32][256];


int usegamma = 2;

extern int gammatable[5][256];


void W_UpdateGamma()
{
	for (int c = 0; c < 256; c++)
	{
		byte r = raw_palette[c][0];
		byte g = raw_palette[c][1];
		byte b = raw_palette[c][2];

		r = gammatable[usegamma][r];
		g = gammatable[usegamma][g];
		b = gammatable[usegamma][b];
		
		palette[c] = fl_rgb_color(r, g, b);
	}
}


void W_LoadPalette()
{
	Lump_c *lump = W_FindLump("PLAYPAL");

	if (! lump)
	{
		FatalError("PLAYPAL lump not found.\n");
		return;
	}

	if (! lump->Seek() ||
		! lump->Read(raw_palette, sizeof(raw_palette)))
	{
		warn("PLAYPAL: read error\n");
		return;
	}

	// find the colour closest to TRANS_PIXEL
	byte tr = raw_palette[TRANS_PIXEL][0];
	byte tg = raw_palette[TRANS_PIXEL][1];
	byte tb = raw_palette[TRANS_PIXEL][2];

	int best_dist = (1 << 30);

	for (int c = 0; c < 256; c++)
	{
		if (c == TRANS_PIXEL)
			continue;

		byte r = raw_palette[c][0];
		byte g = raw_palette[c][1];
		byte b = raw_palette[c][2];

		int sr = (int)r - (int)tr;
		int sg = (int)g - (int)tg;
		int sb = (int)b - (int)tb;

		int dist = sr*sr + sg*sg + sb*sb;

		if (dist < best_dist)
		{
			best_dist = dist;
			trans_replace = c;
		}
	}

	W_UpdateGamma();
}


void W_LoadColormap()
{
	Lump_c *lump = W_FindLump("COLORMAP");

	if (! lump)
	{
		FatalError("COLORMAP lump not found.\n");
		return;
	}

	if (! lump->Seek() ||
		! lump->Read(raw_colormap, sizeof(raw_colormap)))
	{
		warn("COLORMAP: read error\n");
		return;
	}
}


//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
