//------------------------------------------------------------------------
//  Main Window
//------------------------------------------------------------------------
//
//  Eureka DOOM Editor
//
//  Copyright (C) 2008-2009 Andrew Apted
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//------------------------------------------------------------------------

#ifndef __UI_CANVAS_H__
#define __UI_CANVAS_H__

#include "editloop.h"
#include "objects.h"
#include "r_grid.h"


class UI_Canvas : public Fl_Widget
{
private:
	Editor_State_c *e;

	bool render3d;

	Objid highlight;

	bool selbox_active;
	int  selbox_x1, selbox_y1;  // map coords
	int  selbox_x2, selbox_y2;

	bool drag_active;
	int drag_start_x, drag_cur_x;
	int drag_start_y, drag_cur_y;

	int rbscroll_x, rbscroll_y;

	// drawing state only
	int map_lx, map_ly;
	int map_hx, map_hy;

public:
	UI_Canvas(int X, int Y, int W, int H, const char *label = NULL);
	virtual ~UI_Canvas();

public:
	// FLTK virtual method for handling input events.
	int handle(int event);

	// FLTK virtual method for resizing.
	void resize(int X, int Y, int W, int H);

	void set_edit_info(Editor_State_c *new_e) { e = new_e; redraw(); }

	void DrawEverything();

	void DrawMap();

	void DrawMapPoint (int mapx, int mapy);
	void DrawMapLine (int mapx1, int mapy1, int mapx2, int mapy2);
	void DrawMapVector (int mapx1, int mapy1, int mapx2, int mapy2);
	void DrawMapArrow (int mapx1, int mapy1, unsigned angle);
	void DrawKnobbyLine (int mapx1, int mapy1, int mapx2, int mapy2);

	void HighlightSet(Objid& obj);
	void HighlightForget();

	void DrawHighlight(int objtype, int objnum, Fl_Color colour, int dx=0, int dy=0);
	void DrawSelection(selection_c *list);

	void SelboxBegin(int mapx, int mapy);
	void SelboxUpdate(int mapx, int mapy);
	void SelboxFinish(int *x1, int *y1, int *x2, int *y2);

	void DragBegin(int mapx, int mapy);
	void DragUpdate(int mapx, int mapy);
	void DragFinish(int *dx, int *dy);

	bool isSelboxActive() { return selbox_active; }
	bool isDragActive()   { return drag_active; }

private:
	// FLTK virtual method for drawing.
	void draw();

	void DrawGrid();
	void DrawMapBounds();
	void DrawVertices();
	void DrawLinedefs();
	void DrawThings();
	void DrawRTS();
	void DrawObjNum(int x, int y, int obj_no, Fl_Color c);

	void SelboxDraw();

	void RightButtonScroll(int mode);

	// convert screen coordinates to map coordinates
	inline int MAPX(int sx) const { return (grid.orig_x + I_ROUND((sx - w()/2 - x()) / grid.Scale)); }
	inline int MAPY(int sy) const { return (grid.orig_y + I_ROUND((h()/2 - sy + y()) / grid.Scale)); }

	// convert map coordinates to screen coordinates
	inline int SCREENX(int mx) const { return (x() + w()/2 + I_ROUND((mx - grid.orig_x) * grid.Scale)); }
	inline int SCREENY(int my) const { return (y() + h()/2 + I_ROUND((grid.orig_y - my) * grid.Scale)); }

	inline bool Vis(int x, int y, int r) const
	{
		return (x+r >= map_lx) && (x-r <= map_hx) &&
		       (y+r >= map_ly) && (y-r <= map_hy);
	}
	inline bool Vis(int x1, int y1, int x2, int y2) const
	{
		return (x2 >= map_lx) && (x1 <= map_hx) &&
		       (y2 >= map_ly) && (y1 <= map_hy);
	}
};


#endif /* __UI_CANVAS_H__ */

//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
